package com.autentia.tutoriales.dynamicjasper;

import java.awt.Color;
import java.io.File;
import java.util.HashMap;
import java.util.Map;

import net.sf.jasperreports.view.JasperDesignViewer;
import net.sf.jasperreports.view.JasperViewer;
import ar.com.fdvs.dj.domain.DynamicReport;
import ar.com.fdvs.dj.domain.Style;
import ar.com.fdvs.dj.domain.builders.ReflectiveReportBuilder;
import ar.com.fdvs.dj.domain.constants.Border;
import ar.com.fdvs.dj.domain.constants.HorizontalAlign;
import ar.com.fdvs.dj.domain.entities.columns.PropertyColumn;

public class SimpleReportTemplate extends ReportBase {

	public static void main(String[] args) throws Exception {
		SimpleReportTemplate reportTemplate=new SimpleReportTemplate();
		reportTemplate.generateReport();
		reportTemplate.exportToJRXML();
  		JasperViewer.viewReport(reportTemplate.jp);	//finally display the report report
  		JasperDesignViewer.viewReportDesign(reportTemplate.jr);

	}

	@Override
	public DynamicReport buildReport() throws Exception {
		// Definimos las columnas del informe
		final String[] columns = {"name","address","zipcode","country"};
		// Definimos el titulo y el logo como parametros del informe
		params =populateParametersMap("Informe en base a una plantilla");
		// Construimos el informe dinámico en funcionde las columnas y la lista de objetos que se pintara
		final DynamicReport dynamicReport = new ReflectiveReportBuilder(getDataOneReportMock(), columns).setTemplateFile(System.getProperty("user.dir")+ "/src/main/resources/dynamicJasperReportTemplate.jrxml").build();
		
		// Definimos un estilo para el detalle de las properties
		Style styleColumns = new Style(); 
		styleColumns.setHorizontalAlign(HorizontalAlign.CENTER);
		styleColumns.setTextColor(Color.BLUE);
		styleColumns.setBorder(Border.PEN_1_POINT);
		
		// Definimos algunas propiedades para las distintas columnas del informe
		for (int i = 0; i < columns.length; i++) {
			String name = columns[i];
			final PropertyColumn propertyColumn = (PropertyColumn)dynamicReport.getColumns().get(i);
			propertyColumn.setTitle(name);
			propertyColumn.setBlankWhenNull(true);
			propertyColumn.setWidth(25);
			propertyColumn.setStyle(styleColumns);
		}
		
		
		return dynamicReport;
	}

	
	private Map<String, String> populateParametersMap(String title) {
		
		final Map<String, String> parameters = new HashMap<String, String>();
		final String pathLogo=System.getProperty("user.dir")+ "/src/main/resources/autentialogo.png";
		
			parameters.put("REPORT_LOGO", new File(pathLogo).getPath());
			parameters.put("REPORT_TITLE", title);
		return parameters;
	}

	
}

